import 'dart:async';

import 'package:firebase_auth/firebase_auth.dart';
import 'package:hqsoft_core/data_base/data_base.dart';
import 'package:hqsoft_core/value_base/value_base.dart';
import 'package:shared_preferences/shared_preferences.dart';

class FireBaseAuthRepo {
  static FireBaseAuthRepo _instance;

  FireBaseAuthRepo._internal();

  factory FireBaseAuthRepo.getInstance() {
    if (_instance == null) {
      _instance = FireBaseAuthRepo._internal();
    }
    return _instance;
  }

  Future<FireBaseAuthUser> getCurrentUser() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    String userId = prefs.getString(FireBaseStorageKeys.USER_ID_KEY);
    String userDisplayName =
        prefs.getString(FireBaseStorageKeys.USER_DISPLAY_NAME_KEY);
    String userPhotoUrl =
        prefs.getString(FireBaseStorageKeys.USER_PHOTO_URL_KEY);
    String fcmToken = prefs.getString(FireBaseStorageKeys.FCM_TOKEN);
    if (userId != null && userDisplayName != null && userPhotoUrl != null) {
      return FireBaseAuthUser(userId, userDisplayName, userPhotoUrl, fcmToken);
    }
    return null;
  }

  static setCurrentUser(FirebaseUser user) async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    String token = user.getIdToken().toString().isEmpty
        ? prefs.getString(FireBaseStorageKeys.FCM_TOKEN)
        : user.getIdToken().toString();
    await prefs
        .setString(FireBaseStorageKeys.USER_ID_KEY, user.uid)
        .then((value) => prefs.setString(
            FireBaseStorageKeys.USER_DISPLAY_NAME_KEY, user.displayName))
        .then((value) => prefs.setString(
            FireBaseStorageKeys.USER_PHOTO_URL_KEY, user.photoUrl))
        .then((value) => prefs.setString(FireBaseStorageKeys.FCM_TOKEN, token));
  }

  static clearCurrentUser() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    await prefs.clear();
  }

  static Future<String> getFCMToken() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    return prefs.getString(FireBaseStorageKeys.FCM_TOKEN);
  }

  static setFCMToken(String token) async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    await prefs.setString(FireBaseStorageKeys.FCM_TOKEN, token);
  }
}
