import 'package:geolocator/geolocator.dart';

class LocationCheck {
  final Geolocator geoLocator = Geolocator()..forceAndroidLocationManager;

  //https://morioh.com/p/832f968ed090
  //https://www.woolha.com/tutorials/flutter-gps-geolocation-tutorial
  //https://pub.dev/packages/geolocator
//  lowest Location is accurate within a distance of 3000m on iOS and 500m on Android.
//  low Location is accurate within a distance of 1000m on iOS and 500m on Android.
//  medium Location is accurate within a distance of 10m on iOS and between 0m and 100m on Android.
//  high Location is accurate within a distance of 10m on iOS and between 0m and 100m on Android.
//  best Location is accurate within a distance of ~0m on iOS and between 0m and 100m on Android.
//  bestForNavigation Location is accuracy is optimized for navigation on iOS and matches LocationAccuracy.best on Android
  // ignore: missing_return
  Future<Position> getCurrentLocation() async {
    try {
      Position position = await geoLocator
          .getCurrentPosition(desiredAccuracy: LocationAccuracy.best)
          .timeout(new Duration(seconds: 5));
      print(position.toString());

      return position;
    } catch (e) {
      print(e);
    }
  }

  // ignore: missing_return
  Future<Placemark> getAddressFromLatLng(Position _currentPosition) async {
    try {
      List<Placemark> p = await geoLocator.placemarkFromCoordinates(
          _currentPosition.latitude, _currentPosition.longitude);

      Placemark place = p[0];

      print("${place.locality}, ${place.postalCode}, ${place.country}");
      return place;
    } catch (e) {
      print(e);
    }
  }

  void checkPermission() {
    geoLocator.checkGeolocationPermissionStatus().then((status) {
      print('status: $status');
    });
    geoLocator
        .checkGeolocationPermissionStatus(
            locationPermission: GeolocationPermission.locationAlways)
        .then((status) {
      print('always status: $status');
    });
    geoLocator.checkGeolocationPermissionStatus(
        locationPermission: GeolocationPermission.locationWhenInUse)
      ..then((status) {
        print('whenInUse status: $status');
      });
  }
}

final locationCheck = LocationCheck();
